
close all
clear all
Nsamples=1e4;

cm2au=1/2.194746305e5; % wavenumber to au
ps2au=1/(9.10938215e-31*0.52917720859e-10^2/1.054571628e-34*1e12); % picosecond to au

% =================== system specification ========================
H0=[0 100;100 500]; 
Nsites=size(H0,1);
H0=[H0 zeros(Nsites,1);zeros(1,Nsites+1)]; % adding the ground state

Mu=[1 0.5 0;1 -0.5 0]; % dipole orientations
[m1,m2]=meshgrid(1:Nsites);
Mu_mat=reshape(dot(Mu(m1,:),Mu(m2,:),2),Nsites,Nsites);
% =================== system specification ========================

% ==================== bath specification =========================
% Drude spectrum
T=300;                                  % Kelvin
lambda=100;                             % cm^-1
omega_c=53;                             % Drude cutoff in cm^-1
% ==================== bath specification =========================

% ==================== propagation detail =========================
% the unit of time is ps
time.tend   = 0.3;                      % total propagation time
time.dt     = 1e-3;                     % length of infinitesimal time interval
time.tstep  = time.tend/time.dt+1;      % number of time steps
time.t      = linspace(0, time.tend, time.tstep);
time.enlarge= 20;                       % appending the time correlation func. by zeros to increase freq. resolution
time.trunc  = 0.1;                      % manual truncation of the correlation func. after certain time
% ==================== propagation detail =========================

fn=['sPI_emission_T' num2str(T) '_l' num2str(lambda) '_t' num2str(time.tend*1e3) '_dt' num2str(time.dt*1e4) '.dat'];


tt=reshape([num2str(H0(:)) repmat(',',(Nsites+1)^2,1)]',1,[]);
tt=tt(1:end-1);
tt=regexprep(tt,' ','');
comm=['./SSE --Temperature ' num2str(T)       ' --Lambda ' num2str(lambda) ...
           ' --Cutoff ' num2str(omega_c)      ' --Time ' num2str(time.tend*1e3) ...
           ' --Nsteps ' num2str(time.tstep-1) ' --Nsites ' num2str(Nsites+1) ...
           ' --Nsamples ' num2str(Nsamples)   ' --Hamiltonian ' tt ...
           ' --Emission 1'];
system(comm);
clear tt

%% gather results
fida=fopen('aux/pops.dat');
rho=fscanf(fida,'%f',[2*Nsites^2+1 inf]);
fclose(fida);

rho(1,:)=[];
rho(1:Nsites^2,:)=rho(1:Nsites^2,:)+1i*rho(Nsites^2+1:end,:);
rho(Nsites^2+1:end,:)=[];
rho=reshape(rho,[Nsites Nsites time.tstep]);

%% add to the pre-existing results
if exist(fn,'file')==2
    fid=fopen(fn);
    Nsamples_old=sscanf(fgetl(fid),'%*s %*s %*s %*s %d');
    rho_old=NaN(1+Nsites^2*2,size(rho,3));
    for kt=1:size(rho,3)
        rho_old(:,kt)=sscanf(fgetl(fid),'%f',1+Nsites^2*2);
    end
    fclose(fid);
    rho_old(1,:)=[];
    rho_old(1:Nsites^2,:)=rho_old(1:Nsites^2,:)+1i*rho_old(Nsites^2+1:end,:);
    rho_old(Nsites^2+1:end,:)=[];
    rho_old=reshape(rho_old,[Nsites Nsites size(rho,3)]);
    
    rho=rho*Nsamples+rho_old*Nsamples_old;
    Nsamples=Nsamples+Nsamples_old;
    rho=rho/Nsamples;
end

%% account for oscillator's strength
It=NaN(time.tstep,1);
for kk=1:time.tstep
    It(kk)=sum(sum(Mu_mat.*rho(:,:,kk)));
end

%% appending zeros to the dipole-dipole correlation function
It=[It;zeros((time.tstep-1)*(time.enlarge-1),1)];
time.tstep=time.tstep+(time.tstep-1)*(time.enlarge-1);
time.t=[time.t time.tend+time.dt:time.dt:time.tend*time.enlarge];

%% truncate (possible) noisy tails and do fft
Itc=It;Itc(time.t>time.trunc)=0;
[Iw,omega] = specFFT(Itc,time);
Iw=squeeze(Iw);

%% plotting the results in time and frequency domains
FS=20;
figure
tind=1:(time.tstep-1)/time.enlarge;
plot(time.t(tind)*1e3,real(It(tind)),time.t(tind)*1e3,imag(It(tind)))
line([1 1]*time.trunc*1e3,[0 1]) % mark the onset of the truncation
xlabel('time (fs)','FontSize',FS)
ylabel('dipole correlation function','FontSize',FS)
set(gca,'FontSize',FS);

figure
plot(omega,Iw)
xl=max([max(H0(:)) lambda]);
set(gca,'xlim',[-3 3]*xl,'FontSize',FS)
set(gca,'FontSize',FS);
xlabel('wavenumber (cm^{-1})','FontSize',FS)
ylabel('emission spectrum','FontSize',FS)

%% saving results for further use
fid=fopen(fn,'w');
fprintf(fid,'total number of samples: %d\n',Nsamples);
for kt=1:size(rho,3)
    fprintf(fid,'%f ',time.t(kt)*1e3);
    tt=rho(:,:,kt);
    fprintf(fid,repmat('%f ',1,Nsites^2),real(tt(:))');
    fprintf(fid,repmat('%f ',1,Nsites^2),imag(tt(:))');
    fprintf(fid,'\n');
end
fclose(fid);
